@@ -1,6 +1,7 @@
 import os
+
 import jinja2
-from flask import Blueprint, Flask, jsonify, request, render_template, make_response
+from flask import Blueprint, Flask, jsonify, make_response, render_template, request
 from flask.views import View
 
 import flask_limiter
@@ -36,6 +37,7 @@ def app():
         default_limits_deduct_when=lambda response: response.status_code == 200,
         default_limits_cost=default_cost,
         application_limits=["5000/hour"],
+        meta_limits=["2/day"],
         headers_enabled=True,
         storage_uri=os.environ.get("FLASK_RATELIMIT_STORAGE_URI", "memory://"),
     )
