@@ -1,10 +1,12 @@
 """
 Flask-Limiter Extension
 """
+
 from __future__ import annotations
 
 import dataclasses
 import datetime
+import functools
 import itertools
 import logging
 import time
@@ -38,7 +40,6 @@ from .typing import (
     Sequence,
     Set,
     Tuple,
-    TypeVar,
     Union,
     cast,
 )
@@ -85,6 +86,13 @@ class Limiter:
     :param application_limits: a variable list of strings or callables
      returning strings for limits that are applied to the entire application
      (i.e a shared limit for all routes)
+    :param application_limits_per_method: whether application limits are applied
+     per method, per route or as a combination of all method per route.
+    :param application_limits_exempt_when: a function that should return
+     True/False to decide if the application limits should be skipped
+    :param application_limits_deduct_when: a function that receives the
+     current :class:`flask.Response` object and returns True/False to decide
+     if a deduction should be made from the application rate limit(s)
     :param application_limits_cost: The cost of a hit to the global application
      limits as an integer or a function that takes no parameters and returns an
      integer (Default: ``1``).
@@ -108,6 +116,15 @@ class Limiter:
      extension is breached. If the function returns an instance of :class:`flask.Response`
      that will be the response embedded into the :exc:`RateLimitExceeded` exception
      raised.
+    :param meta_limits: a variable list of strings or callables
+     returning strings for limits that are used to control the upper limit of
+     a requesting client hitting any configured rate limit. Once a meta limit is
+     exceeded all subsequent requests will raise a :class:`~flask_limiter.RateLimitExceeded`
+     for the duration of the meta limit window.
+    :param on_meta_breach: a function that will be called when a meta limit in this
+     extension is breached. If the function returns an instance of :class:`flask.Response`
+     that will be the response embedded into the :exc:`RateLimitExceeded` exception
+     raised.
     :param in_memory_fallback: a variable list of strings or callables
      returning strings denoting fallback limits to apply when the storage is
      down.
@@ -135,6 +152,11 @@ class Limiter:
         ] = None,
         default_limits_cost: Optional[Union[int, Callable[[], int]]] = None,
         application_limits: Optional[List[Union[str, Callable[[], str]]]] = None,
+        application_limits_per_method: Optional[bool] = None,
+        application_limits_exempt_when: Optional[Callable[[], bool]] = None,
+        application_limits_deduct_when: Optional[
+            Callable[[flask.wrappers.Response], bool]
+        ] = None,
         application_limits_cost: Optional[Union[int, Callable[[], int]]] = None,
         headers_enabled: Optional[bool] = None,
         header_name_mapping: Optional[Dict[HeaderNames, str]] = None,
@@ -147,6 +169,10 @@ class Limiter:
         on_breach: Optional[
             Callable[[RequestLimit], Optional[flask.wrappers.Response]]
         ] = None,
+        meta_limits: Optional[List[Union[str, Callable[[], str]]]] = None,
+        on_meta_breach: Optional[
+            Callable[[RequestLimit], Optional[flask.wrappers.Response]]
+        ] = None,
         in_memory_fallback: Optional[List[str]] = None,
         in_memory_fallback_enabled: Optional[bool] = None,
         retry_after: Optional[str] = None,
@@ -163,17 +189,16 @@ class Limiter:
         self._default_limits_exempt_when = default_limits_exempt_when
         self._default_limits_deduct_when = default_limits_deduct_when
         self._default_limits_cost = default_limits_cost
+        self._application_limits_per_method = application_limits_per_method
+        self._application_limits_exempt_when = application_limits_exempt_when
+        self._application_limits_deduct_when = application_limits_deduct_when
         self._application_limits_cost = application_limits_cost
         self._in_memory_fallback = []
         self._in_memory_fallback_enabled = in_memory_fallback_enabled or (
             in_memory_fallback and len(in_memory_fallback) > 0
         )
-        self._route_exemptions: Dict[str, ExemptionScope] = defaultdict(
-            lambda: ExemptionScope.NONE
-        )
-        self._blueprint_exemptions: Dict[str, ExemptionScope] = defaultdict(
-            lambda: ExemptionScope.NONE
-        )
+        self._route_exemptions: Dict[str, ExemptionScope] = {}
+        self._blueprint_exemptions: Dict[str, ExemptionScope] = {}
         self._request_filters: List[Callable[[], bool]] = []
 
         self._headers_enabled = headers_enabled
@@ -186,6 +211,7 @@ class Limiter:
         self._swallow_errors = swallow_errors
         self._fail_on_first_breach = fail_on_first_breach
         self._on_breach = on_breach
+        self._on_meta_breach = on_meta_breach
 
         self._key_func = key_func
         self._key_prefix = key_prefix
@@ -217,6 +243,20 @@ class Limiter:
             else []
         )
 
+        self._meta_limits = (
+            [
+                LimitGroup(
+                    limit_provider=limit,
+                    key_function=self._key_func,
+                    scope="meta",
+                    shared=True,
+                )
+                for limit in meta_limits
+            ]
+            if meta_limits
+            else []
+        )
+
         if in_memory_fallback:
             for limit in in_memory_fallback:
                 self._in_memory_fallback.append(
@@ -345,6 +385,18 @@ class Limiter:
         self._application_limits_cost = self._application_limits_cost or config.get(
             ConfigVars.APPLICATION_LIMITS_COST, 1
         )
+        if self._application_limits_per_method is None:
+            self._application_limits_per_method = bool(
+                config.get(ConfigVars.APPLICATION_LIMITS_PER_METHOD, False)
+            )
+        self._application_limits_exempt_when = (
+            self._application_limits_exempt_when
+            or config.get(ConfigVars.APPLICATION_LIMITS_EXEMPT_WHEN)
+        )
+        self._application_limits_deduct_when = (
+            self._application_limits_deduct_when
+            or config.get(ConfigVars.APPLICATION_LIMITS_DEDUCT_WHEN)
+        )
 
         if not self.limit_manager._application_limits and app_limits:
             self.limit_manager.set_application_limits(
@@ -354,6 +406,9 @@ class Limiter:
                         key_function=self._key_func,
                         scope="global",
                         shared=True,
+                        per_method=self._application_limits_per_method,
+                        exempt_when=self._application_limits_exempt_when,
+                        deduct_when=self._application_limits_deduct_when,
                         cost=self._application_limits_cost,
                     )
                 ]
@@ -362,6 +417,9 @@ class Limiter:
             app_limits = self.limit_manager._application_limits
             for group in app_limits:
                 group.cost = self._application_limits_cost
+                group.per_method = self._application_limits_per_method
+                group.exempt_when = self._application_limits_exempt_when
+                group.deduct_when = self._application_limits_deduct_when
             self.limit_manager.set_application_limits(app_limits)
 
         conf_limits = config.get(ConfigVars.DEFAULT_LIMITS, None)
@@ -387,6 +445,23 @@ class Limiter:
                 group.deduct_when = self._default_limits_deduct_when
                 group.cost = self._default_limits_cost
             self.limit_manager.set_default_limits(default_limit_groups)
+
+        meta_limits = config.get(ConfigVars.META_LIMITS, None)
+        if not self._meta_limits and meta_limits:
+            self._meta_limits = [
+                LimitGroup(
+                    limit_provider=meta_limits,
+                    key_function=self._key_func,
+                    scope="meta",
+                    shared=True,
+                )
+            ]
+
+        self._on_breach = self._on_breach or config.get(ConfigVars.ON_BREACH, None)
+        self._on_meta_breach = self._on_meta_breach or config.get(
+            ConfigVars.ON_META_BREACH, None
+        )
+
         self.__configure_fallbacks(app, self._strategy)
 
         if self not in app.extensions.setdefault("limiter", set()):
@@ -572,35 +647,40 @@ class Limiter:
         self,
         obj: flask.Blueprint,
         *,
-        flags: ExemptionScope = ExemptionScope.APPLICATION | ExemptionScope.DEFAULT,
-    ) -> flask.Blueprint:
-        ...
+        flags: ExemptionScope = ExemptionScope.APPLICATION
+        | ExemptionScope.DEFAULT
+        | ExemptionScope.META,
+    ) -> flask.Blueprint: ...
 
     @overload
     def exempt(
         self,
         obj: Callable[..., R],
         *,
-        flags: ExemptionScope = ExemptionScope.APPLICATION | ExemptionScope.DEFAULT,
-    ) -> Callable[..., R]:
-        ...
+        flags: ExemptionScope = ExemptionScope.APPLICATION
+        | ExemptionScope.DEFAULT
+        | ExemptionScope.META,
+    ) -> Callable[..., R]: ...
 
     @overload
     def exempt(
         self,
         *,
-        flags: ExemptionScope = ExemptionScope.APPLICATION | ExemptionScope.DEFAULT,
+        flags: ExemptionScope = ExemptionScope.APPLICATION
+        | ExemptionScope.DEFAULT
+        | ExemptionScope.META,
     ) -> Union[
         Callable[[Callable[P, R]], Callable[P, R]],
         Callable[[flask.Blueprint], flask.Blueprint],
-    ]:
-        ...
+    ]: ...
 
     def exempt(
         self,
         obj: Optional[Union[Callable[..., R], flask.Blueprint]] = None,
         *,
-        flags: ExemptionScope = ExemptionScope.APPLICATION | ExemptionScope.DEFAULT,
+        flags: ExemptionScope = ExemptionScope.APPLICATION
+        | ExemptionScope.DEFAULT
+        | ExemptionScope.META,
     ) -> Union[
         Callable[..., R],
         flask.Blueprint,
@@ -613,7 +693,7 @@ class Limiter:
 
         :param obj: view function or blueprint to mark as exempt.
         :param flags: Controls the scope of the exemption. By default
-         application wide limits and defaults configured on the extension
+         application wide limits, defaults configured on the extension and meta limits
          are opted out of. Additional flags can be used to control the behavior
          when :paramref:`obj` is a Blueprint that is nested under another Blueprint
          or has other Blueprints nested under it (See :ref:`recipes:nested blueprints`)
@@ -650,13 +730,8 @@ class Limiter:
         elif obj:
             self.limit_manager.add_route_exemption(get_qualified_name(obj), flags)
         else:
-            _R = TypeVar("_R")
-            _WO = TypeVar("_WO", Callable[..., _R], flask.Blueprint)
+            return functools.partial(self.exempt, flags=flags)
 
-            def wrapper(obj: _WO) -> _WO:
-                return self.exempt(obj, flags=flags)
-
-            return wrapper
         return obj
 
     def request_filter(self, fn: Callable[[], bool]) -> Callable[[], bool]:
@@ -892,7 +967,7 @@ class Limiter:
         return bool(
             not endpoint
             or not (self.enabled and self.initialized)
-            or endpoint == "static"
+            or endpoint.split(".")[-1] == "static"
             or any(fn() for fn in self._request_filters)
         )
 
@@ -951,6 +1026,37 @@ class Limiter:
         failed_limits: List[Tuple[Limit, List[str]]] = []
         limit_for_header: Optional[RequestLimit] = None
         view_limits: List[RequestLimit] = []
+        meta_limits = list(itertools.chain(*self._meta_limits))
+        if not (
+            ExemptionScope.META
+            & self.limit_manager.exemption_scope(
+                flask.current_app, endpoint, flask.request.blueprint
+            )
+        ):
+            for lim in meta_limits:
+                limit_key, scope = lim.key_func(), lim.scope_for(endpoint, None)
+                args = [limit_key, scope]
+                if not self.limiter.test(lim.limit, *args, cost=lim.cost):
+                    breached_meta_limit = RequestLimit(
+                        self, lim.limit, args, True, lim.shared
+                    )
+                    self.context.view_rate_limit = breached_meta_limit
+                    self.context.view_rate_limits = [breached_meta_limit]
+                    meta_breach_response = None
+                    if self._on_meta_breach:
+                        try:
+                            cb_response = self._on_meta_breach(breached_meta_limit)
+                            if isinstance(cb_response, flask.wrappers.Response):
+                                meta_breach_response = cb_response
+                        except Exception as err:  # noqa
+                            if self._swallow_errors:
+                                self.logger.exception(
+                                    "on_meta_breach callback failed with error %s", err
+                                )
+                            else:
+                                raise err
+                    raise RateLimitExceeded(lim, response=meta_breach_response)
+
         for lim in sorted(limits, key=lambda x: x.limit):
             if lim.is_exempt or lim.method_exempt:
                 continue
@@ -975,7 +1081,7 @@ class Limiter:
                 method = self.limiter.test
             else:
                 method = self.limiter.hit
-                kwargs["cost"] = lim.cost
+            kwargs["cost"] = lim.cost
 
             request_limit = RequestLimit(self, lim.limit, args, False, lim.shared)
             view_limits.append(request_limit)
@@ -1022,6 +1128,11 @@ class Limiter:
                         else:
                             raise err
         if failed_limits:
+            for lim in meta_limits:
+                limit_scope = lim.scope_for(endpoint, flask.request.method)
+                limit_key = lim.key_func()
+                args = [limit_key, limit_scope]
+                self.limiter.hit(lim.limit, *args)
             raise RateLimitExceeded(
                 sorted(failed_limits, key=lambda x: x[0].limit)[0][0],
                 response=on_breach_response,
@@ -1144,16 +1255,13 @@ class LimitDecorator:
         exc_type: Optional[Type[BaseException]],
         exc_value: Optional[BaseException],
         traceback: Optional[TracebackType],
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @overload
-    def __call__(self, obj: Callable[P, R]) -> Callable[P, R]:
-        ...
+    def __call__(self, obj: Callable[P, R]) -> Callable[P, R]: ...
 
     @overload
-    def __call__(self, obj: flask.Blueprint) -> None:
-        ...
+    def __call__(self, obj: flask.Blueprint) -> None: ...
 
     def __call__(
         self, obj: Union[Callable[P, R], flask.Blueprint]
@@ -1187,9 +1295,7 @@ class LimitDecorator:
                         in_middleware=False, callable_name=name
                     )
 
-                return cast(
-                    R, flask.current_app.ensure_sync(cast(Callable[P, R], obj))(*a, **k)
-                )
+                return cast(R, flask.current_app.ensure_sync(obj)(*a, **k))
 
             # mark this wrapper as wrapped by a decorator from the limiter
             # from which the decorator was created. This ensures that stacked
