@@ -1,6 +1,4 @@
-"""
-
-"""
+""" """
 
 import logging
 import time
@@ -201,6 +199,20 @@ def test_default_limit_with_conditional_deduction(extension_factory):
             assert cli.get("/t1/2").status_code == 400
 
 
+def test_deduct_when_custom_cost_moving_window(extension_factory):
+    app, limiter = extension_factory(strategy="moving-window")
+
+    @app.route("/")
+    @limiter.limit("20/minute", cost=15, deduct_when=lambda r: r.status_code == 200)
+    def root():
+        return "root"
+
+    with hiro.Timeline():
+        with app.test_client() as cli:
+            assert cli.get("/").status_code == 200
+            assert cli.get("/").status_code == 429
+
+
 def test_key_func(extension_factory):
     app, limiter = extension_factory()
 
@@ -912,6 +924,11 @@ def test_meta_limits(extension_factory):
     def root():
         return "root"
 
+    @app.route("/exempt")
+    @limiter.exempt
+    def exempt():
+        return "exempt"
+
     with hiro.Timeline().freeze() as timeline:
         with app.test_client() as cli:
             for _ in range(2):
@@ -922,10 +939,12 @@ def test_meta_limits(extension_factory):
 
             # blocked because of max 2 breaches/minute
             assert cli.get("/").status_code == 429
+            assert cli.get("/exempt").status_code == 200
             timeline.forward(59)
             assert cli.get("/").status_code == 200
             assert cli.get("/").status_code == 200
             assert cli.get("/").status_code == 429
+            assert cli.get("/exempt").status_code == 200
             timeline.forward(59)
             # blocked because of max 3 breaches/hour
             response = cli.get("/")
@@ -933,6 +952,7 @@ def test_meta_limits(extension_factory):
             assert response.status_code == 429
             assert response.headers.get("X-RateLimit-Limit") == "3"
             assert response.headers.get("X-RateLimit-Remaining") == "0"
+            assert cli.get("/exempt").status_code == 200
 
             # forward to 1 hour since start
             timeline.forward(60 * 58)
